"use strict";
var path = require("path");
var shell = require("shelljs");
var util = require("util");
var xcode = require("xcode");
var constants = require("./../constants");
var helpers = require("./../common/helpers");
var options = require("../common/options");
var IOSProjectService = (function () {
    function IOSProjectService($projectData, $fs, $childProcess, $errors, $logger, $iOSEmulatorServices, $npm) {
        this.$projectData = $projectData;
        this.$fs = $fs;
        this.$childProcess = $childProcess;
        this.$errors = $errors;
        this.$logger = $logger;
        this.$iOSEmulatorServices = $iOSEmulatorServices;
        this.$npm = $npm;
    }
    Object.defineProperty(IOSProjectService.prototype, "platformData", {
        get: function () {
            var projectRoot = path.join(this.$projectData.platformsDir, "ios");
            return {
                frameworkPackageName: "tns-ios",
                normalizedPlatformName: "iOS",
                appDestinationDirectoryPath: path.join(projectRoot, this.$projectData.projectName),
                appResourcesDestinationDirectoryPath: path.join(projectRoot, this.$projectData.projectName, "Resources", "icons"),
                platformProjectService: this,
                emulatorServices: this.$iOSEmulatorServices,
                projectRoot: projectRoot,
                deviceBuildOutputPath: path.join(projectRoot, "build", "device"),
                emulatorBuildOutputPath: path.join(projectRoot, "build", "emulator"),
                validPackageNamesForDevice: [
                    this.$projectData.projectName + ".ipa"
                ],
                validPackageNamesForEmulator: [
                    this.$projectData.projectName + ".app"
                ],
                frameworkFilesExtensions: [".a", ".framework", ".bin"],
                frameworkDirectoriesExtensions: [".framework"],
                frameworkDirectoriesNames: ["Metadata"],
                targetedOS: ['darwin']
            };
        },
        enumerable: true,
        configurable: true
    });
    IOSProjectService.prototype.validate = function () {
        var _this = this;
        return (function () {
            try {
                _this.$childProcess.exec("which xcodebuild").wait();
            }
            catch (error) {
                _this.$errors.fail("Xcode is not installed. Make sure you have Xcode installed and added to your PATH");
            }
            var xcodeBuildVersion = _this.$childProcess.exec("xcodebuild -version | head -n 1 | sed -e 's/Xcode //'").wait();
            var splitedXcodeBuildVersion = xcodeBuildVersion.split(".");
            if (splitedXcodeBuildVersion.length === 3) {
                xcodeBuildVersion = util.format("%s.%s", splitedXcodeBuildVersion[0], splitedXcodeBuildVersion[1]);
            }
            if (helpers.versionCompare(xcodeBuildVersion, IOSProjectService.XCODEBUILD_MIN_VERSION) < 0) {
                _this.$errors.fail("NativeScript can only run in Xcode version %s or greater", IOSProjectService.XCODEBUILD_MIN_VERSION);
            }
        }).future()();
    };
    IOSProjectService.prototype.createProject = function (projectRoot, frameworkDir) {
        var _this = this;
        return (function () {
            if (options.symlink) {
                _this.$fs.ensureDirectoryExists(path.join(projectRoot, IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER)).wait();
                var xcodeProjectName = util.format("%s.xcodeproj", IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER);
                shell.cp("-R", path.join(frameworkDir, IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER, "*"), path.join(projectRoot, IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER));
                shell.cp("-R", path.join(frameworkDir, xcodeProjectName), projectRoot);
                var directoryContent = _this.$fs.readDirectory(frameworkDir).wait();
                var frameworkFiles = _.difference(directoryContent, [IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER, xcodeProjectName]);
                _.each(frameworkFiles, function (file) {
                    _this.$fs.symlink(path.join(frameworkDir, file), path.join(projectRoot, file)).wait();
                });
            }
            else {
                shell.cp("-R", path.join(frameworkDir, "*"), projectRoot);
            }
        }).future()();
    };
    IOSProjectService.prototype.interpolateData = function (projectRoot) {
        var _this = this;
        return (function () {
            var infoPlistFilePath = path.join(projectRoot, IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER, util.format("%s-%s", IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER, "Info.plist"));
            shell.sed('-i', "__CFBUNDLEIDENTIFIER__", _this.$projectData.projectId, infoPlistFilePath);
            _this.replaceFileName("-Info.plist", path.join(projectRoot, IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER)).wait();
            _this.replaceFileName("-Prefix.pch", path.join(projectRoot, IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER)).wait();
            _this.replaceFileName(IOSProjectService.XCODE_PROJECT_EXT_NAME, projectRoot).wait();
            var pbxprojFilePath = path.join(projectRoot, _this.$projectData.projectName + IOSProjectService.XCODE_PROJECT_EXT_NAME, "project.pbxproj");
            _this.replaceFileContent(pbxprojFilePath).wait();
        }).future()();
    };
    IOSProjectService.prototype.afterCreateProject = function (projectRoot) {
        var _this = this;
        return (function () {
            _this.$fs.rename(path.join(projectRoot, IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER), path.join(projectRoot, _this.$projectData.projectName)).wait();
        }).future()();
    };
    IOSProjectService.prototype.buildProject = function (projectRoot) {
        var _this = this;
        return (function () {
            var basicArgs = [
                "-project",
                path.join(projectRoot, _this.$projectData.projectName + ".xcodeproj"),
                "-target",
                _this.$projectData.projectName,
                "-configuration",
                options.release ? "Release" : "Debug",
                "build",
                'SHARED_PRECOMPS_DIR=' + path.join(projectRoot, 'build', 'sharedpch')
            ];
            var args = [];
            if (options.forDevice) {
                args = basicArgs.concat([
                    "-xcconfig",
                    path.join(projectRoot, _this.$projectData.projectName, "build.xcconfig"),
                    "-sdk",
                    "iphoneos",
                    'ARCHS=armv7 arm64',
                    'VALID_ARCHS=armv7 arm64',
                    "CONFIGURATION_BUILD_DIR=" + path.join(projectRoot, "build", "device")
                ]);
            }
            else {
                args = basicArgs.concat([
                    "-sdk",
                    "iphonesimulator",
                    "-arch",
                    "i386",
                    "VALID_ARCHS=\"i386\"",
                    "CONFIGURATION_BUILD_DIR=" + path.join(projectRoot, "build", "emulator")
                ]);
            }
            _this.$childProcess.spawnFromEvent("xcodebuild", args, "exit", { cwd: options, stdio: 'inherit' }).wait();
            if (options.forDevice) {
                var buildOutputPath = path.join(projectRoot, "build", "device");
                var xcrunArgs = [
                    "-sdk",
                    "iphoneos",
                    "PackageApplication",
                    "-v",
                    path.join(buildOutputPath, _this.$projectData.projectName + ".app"),
                    "-o",
                    path.join(buildOutputPath, _this.$projectData.projectName + ".ipa")
                ];
                _this.$childProcess.spawnFromEvent("xcrun", xcrunArgs, "exit", { cwd: options, stdio: 'inherit' }).wait();
            }
        }).future()();
    };
    IOSProjectService.prototype.isPlatformPrepared = function (projectRoot) {
        return this.$fs.exists(path.join(projectRoot, this.$projectData.projectName, constants.APP_FOLDER_NAME));
    };
    IOSProjectService.prototype.addLibrary = function (platformData, libraryPath) {
        var _this = this;
        return (function () {
            _this.validateDynamicFramework(libraryPath).wait();
            var umbrellaHeader = _this.getUmbrellaHeaderFromDynamicFramework(libraryPath).wait();
            var frameworkName = path.basename(libraryPath, path.extname(libraryPath));
            var targetPath = path.join(_this.$projectData.projectDir, "lib", platformData.normalizedPlatformName, frameworkName);
            _this.$fs.ensureDirectoryExists(targetPath).wait();
            shell.cp("-R", libraryPath, targetPath);
            var pbxProjPath = path.join(platformData.projectRoot, _this.$projectData.projectName + ".xcodeproj", "project.pbxproj");
            var project = new xcode.project(pbxProjPath);
            project.parseSync();
            project.addFramework(path.join(targetPath, frameworkName + ".framework"), { customFramework: true, embed: true });
            project.updateBuildProperty("IPHONEOS_DEPLOYMENT_TARGET", "8.0");
            _this.$fs.writeFile(pbxProjPath, project.writeSync()).wait();
            _this.$logger.info("The iOS Deployment Target is now 8.0 in order to support Cocoa Touch Frameworks.");
        }).future()();
    };
    IOSProjectService.prototype.canUpdatePlatform = function (currentVersion, newVersion) {
        var _this = this;
        return (function () {
            var currentXcodeProjectFile = _this.buildPathToXcodeProjectFile(currentVersion);
            var currentXcodeProjectFileContent = _this.$fs.readFile(currentXcodeProjectFile).wait();
            var newXcodeProjectFile = _this.buildPathToXcodeProjectFile(newVersion);
            var newXcodeProjectFileContent = _this.$fs.readFile(newXcodeProjectFile).wait();
            return currentXcodeProjectFileContent === newXcodeProjectFileContent;
        }).future()();
    };
    IOSProjectService.prototype.updatePlatform = function (currentVersion, newVersion) {
        var _this = this;
        return (function () {
            var sourceFile = path.join(_this.platformData.projectRoot, util.format("%s.xcodeproj", _this.$projectData.projectName));
            var destinationFile = path.join(options.profileDir, "xcodeproj");
            _this.$fs.deleteDirectory(destinationFile).wait();
            shell.cp("-R", path.join(sourceFile, "*"), destinationFile);
            _this.$logger.info("Backup file %s at location %s", sourceFile, destinationFile);
            _this.$fs.deleteDirectory(path.join(_this.platformData.projectRoot, util.format("%s.xcodeproj", _this.$projectData.projectName))).wait();
            var cachedPackagePath = path.join(_this.$npm.getCachedPackagePath(_this.platformData.frameworkPackageName, newVersion), constants.PROJECT_FRAMEWORK_FOLDER_NAME, util.format("%s.xcodeproj", IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER));
            shell.cp("-R", path.join(cachedPackagePath, "*"), path.join(_this.platformData.projectRoot, util.format("%s.xcodeproj", _this.$projectData.projectName)));
            _this.$logger.info("Copied from %s at %s.", cachedPackagePath, _this.platformData.projectRoot);
            var pbxprojFilePath = path.join(_this.platformData.projectRoot, _this.$projectData.projectName + IOSProjectService.XCODE_PROJECT_EXT_NAME, "project.pbxproj");
            _this.replaceFileContent(pbxprojFilePath).wait();
        }).future()();
    };
    IOSProjectService.prototype.buildPathToXcodeProjectFile = function (version) {
        return path.join(this.$npm.getCachedPackagePath(this.platformData.frameworkPackageName, version), constants.PROJECT_FRAMEWORK_FOLDER_NAME, util.format("%s.xcodeproj", IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER), "project.pbxproj");
    };
    IOSProjectService.prototype.validateDynamicFramework = function (libraryPath) {
        var _this = this;
        return (function () {
            var infoPlistPath = path.join(libraryPath, "Info.plist");
            if (!_this.$fs.exists(infoPlistPath).wait()) {
                _this.$errors.failWithoutHelp("The bundle at %s does not contain an Info.plist file.", libraryPath);
            }
            var packageType = _this.$childProcess.exec('/usr/libexec/PlistBuddy -c "Print :CFBundlePackageType" ' + infoPlistPath).wait().trim();
            if (packageType !== "FMWK") {
                _this.$errors.failWithoutHelp("The bundle at %s does not appear to be a dynamic framework.", libraryPath);
            }
        }).future()();
    };
    IOSProjectService.prototype.getUmbrellaHeaderFromDynamicFramework = function (libraryPath) {
        var _this = this;
        return (function () {
            var modulemapPath = path.join(libraryPath, "Modules", "module.modulemap");
            if (!_this.$fs.exists(modulemapPath).wait()) {
                _this.$errors.failWithoutHelp("The framework at %s does not contain a module.modulemap file.", modulemapPath);
            }
            var modulemap = _this.$fs.readText(modulemapPath).wait();
            var umbrellaRegex = /umbrella header "(.+\.h)"/g;
            var match = umbrellaRegex.exec(modulemap);
            if (!match) {
                _this.$errors.failWithoutHelp("The modulemap at %s does not specify an umbrella header.", modulemapPath);
            }
            return match[1];
        }).future()();
    };
    IOSProjectService.prototype.replaceFileContent = function (file) {
        var _this = this;
        return (function () {
            var fileContent = _this.$fs.readText(file).wait();
            var replacedContent = helpers.stringReplaceAll(fileContent, IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER, _this.$projectData.projectName);
            _this.$fs.writeFile(file, replacedContent).wait();
        }).future()();
    };
    IOSProjectService.prototype.replaceFileName = function (fileNamePart, fileRootLocation) {
        var _this = this;
        return (function () {
            var oldFileName = IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER + fileNamePart;
            var newFileName = _this.$projectData.projectName + fileNamePart;
            _this.$fs.rename(path.join(fileRootLocation, oldFileName), path.join(fileRootLocation, newFileName)).wait();
        }).future()();
    };
    IOSProjectService.XCODE_PROJECT_EXT_NAME = ".xcodeproj";
    IOSProjectService.XCODEBUILD_MIN_VERSION = "6.0";
    IOSProjectService.IOS_PROJECT_NAME_PLACEHOLDER = "__PROJECT_NAME__";
    return IOSProjectService;
})();
$injector.register("iOSProjectService", IOSProjectService);
